/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.money

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.items
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarDuration
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController

@Composable
fun MoneyListScreen(
  navController: NavController,
  viewModel: MoneyListViewModel,
  topBar: @Composable () -> Unit,
  bottomBar: @Composable () -> Unit,
) {
  val account by viewModel.account.collectAsState()
  val snackbarHostState = remember { SnackbarHostState() }

  LaunchedEffect(account) {
    val offline = viewModel.checkOffline(account?.server)
    snackbarHostState.currentSnackbarData?.dismiss()
    if (offline) {
      snackbarHostState.showSnackbar(
        message = "Unable to connect to server",
        duration = SnackbarDuration.Indefinite
      )
    }
  }

  Scaffold(
    topBar = topBar,
    bottomBar = bottomBar,
    snackbarHost = {
      SnackbarHost(hostState = snackbarHostState)
    }
  ) { paddingValues: PaddingValues ->
    Column {
      LazyVerticalGrid(
        GridCells.Adaptive(120.dp),
        modifier = Modifier.padding(paddingValues),
        contentPadding = PaddingValues(12.dp),
        horizontalArrangement = Arrangement.SpaceBetween,
      ) {
        items(viewModel.money) { monetaryAmount ->
          MoneyTile(monetaryAmount) {
            viewModel.deposit(it, navController::navigateUp)
          }
        }
      }
    }
  }
}
